"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Emitter_1 = require("./Emitter");
/**
 *
 * @private
 * @template T
 * @param {T} obj
 * @param {Emitter} bus
 * @param {string[]} [path=[]]
 * @returns {T}
 */
function observeObject(obj, bus, path = []) {
    for (const key in obj) {
        if (typeof obj[key] === 'object' && obj[key] !== null) {
            if (Array.isArray(obj[key])) {
                // @ts-ignore
                obj[key] = observeArray(obj[key], bus, path.concat([key]));
            }
            else {
                // @ts-ignore
                obj[key] = observeObject(obj[key], bus, path.concat([key]));
            }
        }
    }
    return new Proxy(obj, {
        set(obj, prop, value) {
            const p = path.concat([prop]);
            console.log('set', obj, p, value);
            if (obj[prop] === undefined) {
                bus.emit('added', p, value);
            }
            else if (value !== obj[prop]) {
                bus.emit('changed', p, value);
            }
            obj[prop] = value;
            return true;
        },
        deleteProperty(obj, prop) {
            bus.emit('deleted', path.concat([prop]));
            const propExists = obj[prop] !== undefined;
            delete obj[prop];
            return propExists;
        }
    });
}
/**
 *
 * @private
 * @template T
 * @param {T} arr
 * @param {Emitter} bus
 * @param {string[]} [path=[]]
 * @returns {T}
 */
function observeArray(arr, bus, path = []) {
    return observeObject(arr, bus, path);
}
/**
 * Returns a proxied object and an emitter which will emit 'added', 'changed' and 'deleted' events
 * whenever changes are made to the returned proxy. Useful for a simple state store.
 *
 ```typescript
 * import { Observable } from 'goodish'
 * const [o, bus] = Observable({ a: 1 })
 *
 * bus.on('added', (path: string[], value: any) => {})
 * bus.on('deleted', (path: string[]) => {})
 * bus.on('changed', (path: string[], value: any) => {})
 *
 * o.b = 2          // Fires 'added' event
 * o.a = 2          // Fires 'changed' event
 * delete o.a       // Fires 'deleted' event
 * o.a = undefined  // Fires 'deleted' event
 * ```
 *
 * @param value
 *
 */
function Observable(value) {
    if (typeof value !== 'object' || value === null) {
        throw new Error('Can only convert objects and arrays to observable objects');
    }
    const bus = new Emitter_1.Emitter();
    const subject = Array.isArray(value) ? observeArray(value, bus) : observeObject(value, bus);
    return [subject, bus];
}
exports.Observable = Observable;
